c     File: grids.f(or)                     ---------------------------|
C                                                                      |
      SUBROUTINE mkgrids
      EXTERNAL psigrids, vgrids
      call psigrids
c                                       generate psi grid
      call vgrids
c                                       generate parallel velocity grid
      call mkqlsm
c                                       generate smoothing function
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE mkqlsm
#include "Implic.inc"
#include "tscunits.inc"
#include "params.inc"
#include "PlPr.inc"
#include "DqlBins.inc"
      INTEGER iv
      REAL
     ^        earg, denomql, qnorm
c     initialize quasilinear smoothing function
      nsmsym = (nsmoo + 1) / 2
      denomql = CONST(1.,0) / AREAL(nsmw)
      do 10 iv = 1, nsmoo
         earg = AREAL(iv - nsmsym) * denomql
         earg = earg * earg / 2.
         qlsm(iv) = exp(-earg)
 10   continue
      call vecnorm(nsmoo, qlsm)
c                                       vecnorm: makes nsmoo values of qlsm 
c                                       add to 1.
      call vsum(nsmoo, qlsm, qnorm)
c                                       vsum: adds nsmoo values of qlsm, 
c                                       returns sum as qnorm.  Should be 1.
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE vecnorm(n, vec)
c                                       Makes n elements of vec add to 1, 
c                                       returns the necessary divisor as norm
#include "Implic.inc"
      INTEGER n
      REAL    vec(n), norm, cnorm
      call vsum(n, vec, norm)
c                                       Adds n elements of vec, returns sum as
c                                       norm
      cnorm = CONST(1.,0) / norm
      call svmult(n, vec, cnorm, vec)
      return
      END
c     ------------------------------------------------------------------       
      SUBROUTINE vsum(n, vec, sum)
c                                       Adds n elements of vec, returns sum
#include "Implic.inc"
      INTEGER n, i
      REAL
     ^     vec(n), sum
      sum = CONST(0.0,0)
      do 10 i = 1, n
         sum = sum + vec(i)
 10   continue
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE svmult(n, VecIn, scal, VecOut)
#include "Implic.inc"
      INTEGER n, i
      REAL
     ^        VecIn(n), scal, VecOut(n)
      do 10 i = 1, n
         VecOut(i) = scal * VecIn(i)
 10   continue
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE psigrids
c                                       generate psi grid
#include "Implic.inc"
#include "params.inc"
#include "RayBins.inc"
#include "RayWrk.inc"
#include "ProfBody.inc"
      EXTERNAL ugrid
      REAL
     ^     TTINY, dpp
      INTEGER l
c, delpsi
      PARAMETER(TTINY = CONST(1.,-5))
      dpp = (psimaxx - psiminx) * TTINY / (AREAL(npsi - 1))
      call ugrid(PsiAry, npsi, psiminx + dpp, psimaxx - dpp)
c                                       ENFORCES INTERPOLATION !!
      delpsi = abs(PsiAry(2) - PsiAry(1))
      do 10 l=1,Npsi-1
        MidAry(l) = 0.5*(PsiAry(l+1)+PsiAry(l))
 10   continue
        MidAry(Npsi) = PsiAry(Npsi)
      call Volcalc
      return
      END
c     ------------------------------------------------------------------      
      SUBROUTINE VolCalc
#include "Implic.inc"
#include "params.inc"
#include "RayBins.inc"
#include "RayWrk.inc"
#include "ProfBody.inc"
#include "TSCgrap.inc"
      EXTERNAL ugrid
      INTEGER ips
      REAL xlookup, yreturn
      ips = Npsi
      iVlAry(ips) = iVlVec(NpsiJ)
c
c     compute integral of volume by interpolation     
      do 10 ips = 1, Npsi-1
        xlookup = MidAry(ips)
        call linr1d(NpsiJ, MidVec , iVlVec, xlookup, yreturn)
        iVlAry(ips) = yreturn
 10   continue
c
c     compute dVol which is centered
        ips =1
        dVol(ips) = iVlAry(ips)
      do 30 ips = 2, Npsi
        dVol(ips) = iVlAry(ips) - iVlAry(ips-1)
 30   continue
      
c        call LSCpause
c        write(6,'('' psi index, centered volume elements,'',
c     ^            '' integrated vol:'')')
c        do 50 ips=1,npsi
c          write(6,'(1x,i4, 1x, 1pe10.3, 4x,1pe10.3)') 
c     ^         ips, dVol(ips), iVlAry(ips)
c 50     continue
c        call LSCpause

      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE blockcpy(n, tovec, nto, frvec, nfrom)
#include "Implic.inc"
      INTEGER n, i, nto, nfrom
      REAL
     ^        tovec(nto, *), frvec(nfrom, *) 
      do 10 i = 1, n
         tovec(1, i) = frvec(1, i)
 10   continue
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE vgrids
#include "Implic.inc"
#include "tscunits.inc"
#include "params.inc"
#include "FeBins.inc"
      EXTERNAL ugrid
c     generate parallel velocity grid
      if (2 * (nv / 2) .eq. nv) then
         call LSCwarn (' require ODD nv ')
         nv = nv - 1
      endif
c                                       set UniformGRID into Vpar
         call ugrid(Vpar, nv, Vmin, Vmax)
c                                       set EXPonentialGRID into Vpar
c        call egrid(Vpar, nv, Vmin, Vmax, StpRange)
c
      call mkdelv(nv, dvsym, Vpar)
c                                       compute forward difference on Vpar,
c                                       and place into dvsym
      call mkdvp(nv, dvplus, Vpar)
      call mkvinv(nv, dvip, dvplus)
      call mkvinv(nv, dvisym, dvsym)
c                                       construct inverse, so dvisym is 1/dv
      IvZero = (nv + 1) / 2
c                                       require nv to be odd !!!!!
      return
      END
c     ------------------------------------------------------------------
      SUBROUTINE MiscInit
#include "Implic.inc"
#include "params.inc"
#include "FeBins.inc"
#include "WkAry.inc"
      REAL       ONE, FLOAToNV
      DATA       ONE    /
     ^     CONST(1.0,0) /
      FLOAToNV = AREAL(nv)
      call ugrid(ilist, nv,ONE, FLOAToNV )
c     call ugrid(ilist, nv, 1., float(nv))
c     generate an integer list (stored in a floating array)
      return
      END
c
c     Sorting by Straight Insertion   begins --------------------------|
c     -                                                                |
c     -                                                                |
      SUBROUTINE PikSrtNR ( n, arr )
#include "Implic.inc"
c     Sorts an array ARR of length N into acending numerical order,
c     by straight instertion.  N is input; ARR is replaced on output
c     by its sorted rearrangement.
c     Numerical Recipes in Fortran by W. H. Press, et al., p 226
      INTEGER n, j, i
      REAL    arr(n), a
      do 12 j = 2, n
        a = arr(j)
        do 11 i = j-1, 1, -1
            if ( arr(i) .le. a ) go to 10
            arr(i+1) = arr(i)
 11     continue
        i = 0
 10     arr(i+1) = a
 12   continue
      return
      END
c
c     ------------------------------------------------------------------
c
      SUBROUTINE PikSr2NR ( n, arr, brr )
#include "Implic.inc"
c     Sorts an array ARR of length N into acending numerical order,
c     by straight instertion --- while making the corresponding
c     rearrangement of the array BRR.
c     N is input; ARR is replaced on output
c     by its sorted rearrangement.  So is BRR.
c     Numerical Recipes in Fortran by W. H. Press, et al., p 228
      INTEGER n, j, i
      REAL    arr(n), brr(n), a, b
      do 12 j = 2, n
        a = arr(j)
        b = brr(j)
        do 11 i = j-1, 1, -1
            if ( arr(i) .le. a ) go to 10
            arr(i+1) = arr(i)
            brr(i+1) = brr(i)
 11     continue
        i = 0
 10     arr(i+1) = a
        brr(i+1) = b
 12   continue
      return
      END
c     -                                                                |
c     -                                                                |
c     Sorting by Straight Insertion   ends ----------------------------|
c
c
C                                                                      |
C                                                                      |
C     File: grids.f(or) ends                ---------------------------|








